
#include "../ldr/idaldr.h"
#include <exehdr.h>
#include <queue.hpp>
#include <vector>
#include "zenldr.h"


//--------------------------------------------------------------------------
static bool bigendian;
static R_HEADER header;
static std::vector<SECTION_INFO> sections;


//--------------------------------------------------------------------------
//
//      check input file format. if recognized, then return 1
//      and fill 'fileformatname'.
//      otherwise return 0
//
int idaapi accept_file(linput_t *li, char fileformatname[MAX_FILE_FORMAT_NAME], int n)
{
    if (n)
        return 0;

    // Read header.
    if (qlread(li, &header, sizeof(header)) != sizeof(header))
        return 0;

    // Check it's tag.
    if (header.Tag == 'EDOC')
    {
        // Adjust the endian.
        header.Size = swap32(header.Size);
        header.Unknown = swap32(header.Unknown);
        bigendian = true;
    }
    else if (header.Tag == 'CODE')
    {
        bigendian = false;
    }
    else
    {
        return 0;
    }

    // Check the size of the file.
    unsigned long size = qlsize(li);
    if (size < (header.Size + R_HEADER::SizeOf))
        return 0;

    qstrncpy(fileformatname, "Zen CODE format", MAX_FILE_FORMAT_NAME);
    if (bigendian)
        set_processor_type("TMS32055", SETPROC_ALL/*|SETPROC_FATAL*/);
    else
        set_processor_type("ARM", SETPROC_ALL/*|SETPROC_FATAL*/);

    return 1;
}

//--------------------------------------------------------------------------
//
//      load file into the database.
//
void idaapi load_file(linput_t *li, ushort /*neflag*/, const char * /*fileformatname*/)
{
    sections.clear();
    qlseek(li, sizeof(R_HEADER), SEEK_CUR);

    do {

        SECTION_INFO info;
        info.FileOffset = qlseek(li, 0, SEEK_CUR);

        if (qlread(li, &info.Section, sizeof(info.Section)) != sizeof(info.Section))
            break;

        if (bigendian)
        {
            info.Section.Address = swap32(info.Section.Address);
            info.Section.Size = swap32(info.Section.Size);
            info.Section.Checksum = swap32(info.Section.Checksum);
        }

        sections.push_back(info);

        qlseek(li, info.FileOffset + sizeof(info.Section) + info.Section.Size);
    } while (true);

    inf.baseaddr = 0;
    inf.lflags |= LFLG_PC_FLAT;

    int nsel = 1;
    for (std::vector<SECTION_INFO>::iterator i = sections.begin();
         i != sections.end(); ++i)
    {
        file2base(li,
                  i->FileOffset + sizeof(i->Section),
                  i->Section.Address,
                  i->Section.Address+i->Section.Size,
                  FILEREG_PATCHABLE);

        set_selector(nsel, 0);
		add_segm(nsel,
				 i->Section.Address,
				 i->Section.Address+i->Section.Size,
				 NULL,
				 CLASS_CODE);
        segment_t* segm = get_segm_by_sel(nsel);
        if (segm != NULL)
		{
            segm->perm = SEGPERM_EXEC | SEGPERM_WRITE | SEGPERM_READ;
            segm->bitness = 1;
            segm->update();
        }
        nsel++;

        add_long_cmt(i->Section.Address, true,
                     "--------------------------------------"
                     "-------------------------------------");
        add_long_cmt(i->Section.Address, true,
                     "Section:\n"
                     "  File offset : %08x\n"
                     "  Address     : %08x\n"
                     "  Size        : %08x\n"
                     "  Checksum    : %08x\n",
                     i->FileOffset, i->Section.Address,
                     i->Section.Size, i->Section.Checksum);
    }
}


//----------------------------------------------------------------------
//
//      LOADER DESCRIPTION BLOCK
//
//----------------------------------------------------------------------
loader_t LDSC =
{
  IDP_INTERFACE_VERSION,
  0,                            // loader flags
//
//      check input file format. if recognized, then return 1
//      and fill 'fileformatname'.
//      otherwise return 0
//
  accept_file,
//
//      load file into the database.
//
  load_file,
//
//	create output file from the database.
//	this function may be absent.
//
  NULL,
//      take care of a moved segment (fix up relocations, for example)
  NULL,
//      initialize user configurable options based on the input file.
  NULL,
};
